"use strict";
/**
  * ******************************************************
  *                                                       *
  *   Copyright (C) Microsoft. All rights reserved.       *
  *                                                       *
  *******************************************************
  */
const fs = require("fs");
const path = require("path");
const vcremoteUtility = require("./util-helper");
var UtilHelper = vcremoteUtility.UtilHelper;
var VCRemoteUtility;
(function (VCRemoteUtility) {
    class ResourcesManager {
        /**
         * Initialize the Resource Manager
         *
         * @param {string} language The default language to look up via getString
         * @param {string} resourcesDir The location to look for resources. The expectation is that this location has subfolders such as "en" and "it-ch" which contain "resources.json"
         */
        static init(language, resourcesDir) {
            if (ResourcesManager.SupportedLanguages.length === 0) {
                // Initialize the resources for this package the first time we are initialized
                ResourcesManager.loadFolder(path.join(__dirname, "../../localization/app/localization/i18n"));
            }
            ResourcesManager.loadFolder(resourcesDir);
            ResourcesManager.DefaultLanguage = ResourcesManager.bestLanguageMatchOrDefault(language);
        }
        // For unit tests
        static teardown() {
            ResourcesManager.Resources = {};
            ResourcesManager.SupportedLanguages = [];
        }
        /** ...optionalArgs is only there for typings, function rest params */
        static getString(id, ...optionalArgs) {
            var args = UtilHelper.getOptionalArgsArrayFromFunctionCall(arguments, 1);
            return ResourcesManager.getStringForLanguage(ResourcesManager.DefaultLanguage, id, args);
        }
        /** ** ...optionalArgs is only there for typings, function rest params** */
        static getStringForLanguage(requestOrAcceptLangs, id, ...optionalArgs) {
            if (!ResourcesManager.Resources) {
                throw new Error("Resources have not been loaded");
            }
            var lang = ResourcesManager.bestLanguageMatchOrDefault(requestOrAcceptLangs);
            var s = ResourcesManager.Resources[lang][id];
            if (!s) {
                return s;
            }
            else if (Array.isArray(s)) {
                // Allow longer resources strings to be specified as a list of strings, which represent multiple lines
                s = s.join("\n");
            }
            /*All args passed to current function:
            you can call getString('foo', 'bar', 'baz') or getString('foo',['bar', 'baz'])
            and the utility function will extract ['bar', 'baz'] as args in both cases*/
            var args = UtilHelper.getOptionalArgsArrayFromFunctionCall(arguments, 2);
            if (args) {
                for (var i = 0; i < args.length; i++) {
                    s = s.replace("{" + i + "}", args[i]);
                }
            }
            return s;
        }
        static bestLanguageMatchOrDefault(requestOrAcceptLangs) {
            var lang = ResourcesManager.bestLanguageMatch(requestOrAcceptLangs);
            if (!ResourcesManager.Resources[lang]) {
                lang = ResourcesManager.DefaultLanguage;
            }
            return lang.toLowerCase();
        }
        /**
         * requestOrAcceptLangs can either be:
         * A string, with format "LangSpec[,LangSpec]*" where LangSpec is "Language[;anything]"
         *   e.g. "pl,fr-FR;q=0.3,en-US;q=0.1" is interpreted as "pl" or "fr-FR" or "en-US". Currently we ignore provided quality (q) values
         * An array, which we assume is an array of strings representing languages such as "pl" or "fr-FR"
         * A (express-style) HTTP Request object, with a headers property specifing "accept-language" in a string, as above
         *
         * This allows us to handle simple cases of a single string, as well as more complex cases where a client specifies
         * multiple preferences.
         */
        static bestLanguageMatch(requestOrAcceptLangs) {
            if (Array.isArray(requestOrAcceptLangs)) {
                return ResourcesManager.getBestLanguageFromArray(requestOrAcceptLangs);
            }
            var langString;
            if (!requestOrAcceptLangs) {
                return ResourcesManager.DefaultLanguage;
            }
            else if (typeof requestOrAcceptLangs === "string") {
                langString = requestOrAcceptLangs;
            }
            else if (requestOrAcceptLangs.headers) {
                langString = requestOrAcceptLangs.headers["accept-language"] || "";
            }
            else {
                throw new Error("Unsupported type of argument for acceptLangs: " + (typeof requestOrAcceptLangs));
            }
            return ResourcesManager.getBestLanguageFromArray(langString.split(",").map(function (l) { return l.split(";")[0]; }));
        }
        /**
         * Given a list of languages, we try to find an exact match if we can, and we fall back to a primary language otherwise.
         *  e.g. "fr-CA" will use "fr-CA" resources if present, but will fall back to "fr" resources if they are available
         */
        static getBestLanguageFromArray(acceptLangs) {
            var primaryLanguageMatch = null;
            for (var i = 0; i < acceptLangs.length; ++i) {
                var lang = acceptLangs[i].toLowerCase();
                var primaryLang = lang.split("-")[0];
                if (ResourcesManager.SupportedLanguages.indexOf(lang) !== -1) {
                    // Exact match on full language header, which could include the region
                    return lang;
                }
                if (ResourcesManager.SupportedLanguages.indexOf(primaryLang) !== -1) {
                    // Match on primary language (e.g. it from it-CH). We may find a better match later, so continue looking.
                    primaryLanguageMatch = primaryLang;
                }
            }
            return primaryLanguageMatch;
        }
        static loadFolder(resourcesDir) {
            fs.readdirSync(resourcesDir).forEach(function (filename) {
                try {
                    var res = ResourcesManager.loadLanguage(filename, resourcesDir);
                    if (ResourcesManager.Resources[filename.toLowerCase()]) {
                        Object.keys(res).forEach(function (key) {
                            ResourcesManager.Resources[filename.toLowerCase()][key] = ResourcesManager.Resources[filename.toLowerCase()][key] || res[key];
                        });
                    }
                    else {
                        ResourcesManager.Resources[filename.toLowerCase()] = res;
                    }
                    ResourcesManager.SupportedLanguages.push(filename.toLowerCase());
                }
                catch (e) {
                    // Folder was not a valid resource; ignore it
                }
            });
        }
        static loadLanguage(language, resourcesDir) {
            var resourcesPath = path.join(resourcesDir, language, "resources.json");
            return require(resourcesPath);
        }
    }
    ResourcesManager.Resources = {};
    ResourcesManager.SupportedLanguages = [];
    ResourcesManager.DefaultLanguage = "en";
    VCRemoteUtility.ResourcesManager = ResourcesManager;
})(VCRemoteUtility || (VCRemoteUtility = {}));
module.exports = VCRemoteUtility;
